clear all
clc

M = 0.5;
m = 0.2;
b = 0.1;
l = 0.3;
I= 1/3*m*(l)^2;
g = 9.8;
p = I*(M+m)+M*m*l^2;  

A = [0      1              0           0;
     0 -(I+m*l^2)*b/p  (m^2*g*l^2)/p   0;
     0      0              0           1;
     0 -(m*l*b)/p       m*g*l*(M+m)/p  0];
B = [ 0; (I+m*l^2)/p; 0; m*l/p];
C = [1 0 0 0; 0 0 1 0];
D = [0; 0];

sistema_tc = ss(A,B,C,D)

%DISCRETIZZAZIONE
Tc=0.02;
sistema_td=c2d(sistema_tc,Tc);
[Ad,Bd,Cd,Dd] = ssdata(sistema_td)

% AUTOVALORI E POLINOMIO CARATTERISTICO 
% DEL SISTEMA DISCRETIZZATO
Autov_Ad=eig(Ad)
PcarAd=poly(Autov_Ad)

%% verifica controllabilit
Mc=[Bd Ad*Bd Ad^2*Bd Ad^3*Bd]
Mc=ctrb(Ad,Bd) % istruzione Matlab alternativa
rankMc=rank(Mc)
 

%% ASSEGNAMENTO POLI MEDIANTE TRASFORMAZIONE IN FORMA COMPAGNA
%matrice di trasformazione nella forma compagna contr.

a3=PcarAd(2);
a2=PcarAd(3);
a1=PcarAd(4);
a0=PcarAd(5);

pdes=[0.95 0.95 0.95 0.95];
Pcardes=poly(pdes)

gamma3=Pcardes(2);
gamma2=Pcardes(3);
gamma1=Pcardes(4);
gamma0=Pcardes(5);

Kbar=[gamma0-a0,gamma1-a1,gamma2-a2,gamma3-a3]

Gamma=[a1 a2 a3 1;a2 a3 1 0;a3 1 0 0;1 0 0 0]
T=Mc*Gamma

%calcolo guadagni del controllo state-feedback
K=Kbar*inv(T)

%verifica autovalori
Autov_ciclochiuso=eig(Ad-Bd*K)


%% CALCOLO FUNZIONI DI TRASFERIMENTO E GUADAGNI

[num den]=ss2tf(Ad-Bd*K,Bd,C,D);
Gry=tf(num(1,:),den,Tc)
Grdx3=tf(num(2,:),den,Tc)

zeri_Gry=roots(num(1,:))
zeri_Grdx3=roots(num(2,:))

mu = dcgain(Gry)
gain_yteta = dcgain(Grdx3)

beta=1/mu

%% FIGURE STATE-FEEDBACK
% usare file PendoloInversoTut07_statefeedback.slx

clear setp
t=Ref.Time;
setp=squeeze(Ref.Data);
y=x.Data(:,1);
teta=x.Data(:,3);
yNL=x_NL.Data(:,1);
tetaNL=x_NL.Data(:,3);

figure(1)
plot(t,[setp y],'LineWidth',2),grid
title('Posizione carrello - modello linearizzato','FontSize',14)
legend('Set-point','y(t)')
set(gca,'FontSize',14)
axis([0 10 -0.5 3.5])

figure(2)
plot(t,[setp yNL],'LineWidth',2),grid
title('Posizione carrello - modello nonlineare','FontSize',14)
legend('Set-point','y(t)')
set(gca,'FontSize',14)
axis([0 10 -0.5 3.5])

figure(3)
plot(t,360/(2*pi)*[teta tetaNL],'LineWidth',2),grid
title('Deviazione dalla verticale [grad]','FontSize',14)
legend('Mod. linearizzato','Mod. non lineare')
set(gca,'FontSize',14)
axis([0 10 -20 10])

figure(4)
plot(t,[setp y yNL],'LineWidth',2),grid
title('Posizione carrello','FontSize',14)
legend('Set-point','Mod. lineare','Mod. nonlineare')
set(gca,'FontSize',14)
axis([0 10 -1 7])

figure(5)
plot(t,360/(2*pi)*[teta tetaNL],'LineWidth',2),grid
title('Deviazione dalla verticale [grad]','FontSize',14)
legend('Mod. linearizzato','Mod. non lineare')
set(gca,'FontSize',14)
axis([0 10 -40 20])

figure(6)
plot(t,[setp y yNL],'LineWidth',2),grid
title('Posizione carrello','FontSize',14)
legend('Set-point','Mod. lineare','Mod. nonlineare')
set(gca,'FontSize',14)
axis([0 15 -1 7])

figure(7)
plot(t,360/(2*pi)*[teta tetaNL],'LineWidth',2),grid
title('Deviazione dalla verticale [grad]','FontSize',14)
legend('Mod. linearizzato','Mod. non lineare')
set(gca,'FontSize',14)
axis([0 15 -15 10])

figure(8)
plot(t,[setp y yNL],'LineWidth',2),grid
title('Posizione carrello','FontSize',14)
legend('Set-point','Mod. lineare','Mod. nonlineare')
set(gca,'FontSize',14)
axis([0 120 -7 7])

figure(9)
plot(t,360/(2*pi)*[teta tetaNL],'LineWidth',2),grid
title('Deviazione dalla verticale [grad]','FontSize',14)
legend('Mod. linearizzato','Mod. non lineare')
set(gca,'FontSize',14)
axis([0 120 -1 0.5])

%% PROGETTO OSSERVATORE DI STATO

%VERIFICA OSSERVABILITA
clc
Mo=[Cd;Cd*Ad;Cd*Ad^2]
rankMo=rank(Mo)

% progetto osservatore con il comando Matlab "place"
p_oss_des=[0.19 0.18 0.17 0.16];
Lo = place(Ad',Cd',p_oss_des)'

%verifica autovalori dinamica errore
Autov_errore=eig(Ad-Lo*Cd)

yzero=0.2; %posizione iniziale del carrello


%% FIGURE OBSERVER-BASED
% usare file PendoloInversoTut07_statefeedback_observer.slx

t=Ref.Time;
that=xhatNL.Time;

setp=squeeze(Ref.Data);
y=x.Data(:,1);
teta=x.Data(:,3);

yNL=x_NL.Data(:,1);
ydotNL=x_NL.Data(:,2);
tetaNL=x_NL.Data(:,3);
tetadotNL=x_NL.Data(:,4);

ydotNLhat=xhatNL.Data(:,2);
tetadotNLhat=xhatNL.Data(:,4);


figure(10)
plot(t,[setp y yNL],'LineWidth',2),grid
title('Posizione carrello','FontSize',14)
legend('Set-point','Mod. lineare','Mod. nonlineare')
set(gca,'FontSize',14)
axis([0 10 -0.5 6.5])

figure(11)
plot(t,360/(2*pi)*[teta tetaNL],'LineWidth',2),grid
title('Deviazione dalla verticale [grad]','FontSize',14)
legend('Mod. linearizzato','Mod. non lineare')
set(gca,'FontSize',14)
axis([0 10 -30 30])

figure(12)
plot(t,[setp y yNL],'LineWidth',2),grid
title('Posizione carrello','FontSize',14)
legend('Set-point','Mod. lineare','Mod. nonlineare')
set(gca,'FontSize',14)
axis([0 120 -7 7])

figure(13)
plot(t,ydotNL,that,ydotNLhat,'LineWidth',2),grid
title('Velocita del carrello. Modello non lineare','FontSize',14)
legend('Vel. misurata','Vel. osservata')
set(gca,'FontSize',14)
axis([0 0.7 -1 7])

figure(14)
plot(t,tetadotNL,that,tetadotNLhat,'LineWidth',2),grid
title('Velocita angolare asta. Mod. non lineare','FontSize',14)
legend('Vel. misurata','Vel. osservata')
set(gca,'FontSize',14)
axis([0 0.15 -0.5 1.5])

%F_NLOBS=F_NL;
figure(15)
plot(that,F_NLOBS.Data,that,F_NL.Data,'LineWidth',2),grid
title('Forza applicata [N]. Mod. non lineare','FontSize',14)
legend('Observer-based','State-feedback')
set(gca,'FontSize',14)
%axis([0 0.2 -5 15])